/*
 *  Created on Nov 3, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.nukesoft.bot2.core;

import java.util.*;

/**
 * The DMConfig class is used to configure the DialogManager.
 * 
 * @author Ryan Michela
 */
public class DMConfig
{
	List cpc = new ArrayList();
	List lpc = new ArrayList();
	List dbc = new ArrayList();
	
	int dialogTimeout;
	int logFlushDelay;
	String adminLineProcBindID;
	String adminEscapeSequence;
	String logWorkingDir;
	
	public DMConfig(int dialogTimeout, 
					int logFlushDelay, 
					String logWorkingDir,
					String adminLineProcBindID,
					String adminEscapeSequence)
	{
		this.dialogTimeout = dialogTimeout;
		this.logFlushDelay = logFlushDelay;
		this.logWorkingDir = logWorkingDir;
		this.adminLineProcBindID = adminLineProcBindID;
		this.adminEscapeSequence = adminEscapeSequence;
	}
	
	/**
	 * Adds a configuration record to the CommPort config table.
	 * @param conf The configuration entry to add.
	 */
	public void addCPConfig(CPConfig conf)
	{
		cpc.add(conf);
	}
	
	/**
	 * Adds a configuration record to the LineProc config table.
	 * @param conf The configuration entry to add.
	 */
	public void addLPConfig(LPConfig conf)
	{
		lpc.add(conf);
	}
	
	/**
	 * Adds a configuration record to the Default Bind config table.
	 * @param conf The configuration entry to add.
	 */	
	public void addDBConfig(DBConfig conf)
	{
		dbc.add(conf);
	}
	
	/**
	 * 
	 * The CPConfig class is used by the DialogManager to configure a CommPort.
	 * 
	 * @author Ryan Michela
	 */
	public class CPConfig
	{
		String bindName;
		String loggin;
		String password;
		String libPath;
		int pingDelay;
		
		public CPConfig(String bindName, String loggin, String password, String libPath, int pingDelay)
		{
			this.bindName = bindName;
			this.loggin = loggin;
			this.password = password;
			this.libPath = libPath;
			this.pingDelay = pingDelay;
		}
	}
	
	/**
	 * 
	 * The LPConfig class is used by the DialogManager to configure a LineProc.
	 * 
	 * @author Ryan Michela
	 */
	public class LPConfig
	{
		String bindName;
		String libPath;
		
		public LPConfig(String bindName, String libPath)
		{
			this.bindName =  bindName;
			this.libPath = libPath;
		}
	}
	
	/**
	 * 
	 * The DBConfig class is used by the DialogManager to configure a Default Bind.
	 * 
	 * @author Ryan Michela
	 */
	public class DBConfig
	{
		String commPortBindName;
		String lineProcBindName;
		
		public DBConfig(String commPortBindName, String lineProcBindName)
		{
			this.commPortBindName = commPortBindName;
			this.lineProcBindName = lineProcBindName;
		}
	}
}
