/*
 *  Created on Nov 10, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.nukesoft.bot2.lineproc;

import java.util.Random;
import org.nukesoft.bot2.core.IM;

/**
 * GenericLineProc is an abstract interpretation of the LineProc interface.
 * It provides stub implementations of all the required methods.
 * 
 * @author Ryan Michela
 */
public abstract class GenericLineProc implements LineProc
{
	/**
	 * A reference to this LineProc's LineProcContext.
	 */
	protected LineProcContext context;
	/**
	 * The dialog bind name of this LineProc's associated
	 * IM client.
	 */
	protected String remoteScreenName;
	/**
	 * The bind ID of the comm lib handleing this dialog.
	 */
	protected String myCommLibBindID;
	
	private Random r = new Random(System.currentTimeMillis());
	
	public GenericLineProc()
	{
		
	}
	
	/**
	 * @see org.nukesoft.bot2.lineproc.LineProc#init(java.lang.String)
	 */
	public void init(String screenName)
	{
		remoteScreenName = screenName;
	}

	/**
	 * @see org.nukesoft.bot2.lineproc.LineProc#bindContext(org.nukesoft.bot2.lineproc.LineProcContext)
	 */
	public void bindContext(LineProcContext context)
	{
		this.context = context;
		String identity = context.getIdentity();
		this.myCommLibBindID = identity.substring(0, identity.indexOf('~'));
	}

	/**
	 * @see org.nukesoft.bot2.lineproc.LineProc#handleIM(org.nukesoft.bot2.core.IM)
	 */
	public abstract void handleIM(IM message);

	/**
	 * @see org.nukesoft.bot2.lineproc.LineProc#handleWarn(java.lang.String)
	 */
	public void handleWarn(String screenName)
	{
	
	}

	/**
	 * @see org.nukesoft.bot2.lineproc.LineProc#shutdown()
	 */
	public void shutdown()
	{
	
	}
	
	/**
	 * Causes this thread to sleep for a few seconds.
	 * Can be used to create the effect of the bot typing a response.
	 * @param minDelay The minimum number of seconds to sleep for.
	 * @param maxDelay The maximum number of seconds to sleep for.
	 */
	protected void delay(int minDelay, int maxDelay)
	{
		//sleep for a few seconds
		int delay = r.nextInt(maxDelay) + minDelay;
		try{Thread.sleep(1000 * delay);}catch(Exception e){}
	}
}
