/*
 *  Created on Oct 16, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.nukesoft.bot2.lineproc;
import org.nukesoft.bot2.core.*;

/**
 * The LineProc interface is the interface implemented by all external
 * line processor libraries. A new line processor instance is loaded for
 * each active conversation. Line processors need not be thread safe and
 * can mantain internal state.
 * 
 * @author Ryan Michela
 */
public interface LineProc
{
	/**
	 * Called as soon as a line processor is loaded. Used to perform any
	 * necesary initialization. It is better to do initialization here
	 * because you have access to the remote screen name of the person doing
	 * the initialization.
	 * @param screenName The name of the dialog that is initializing the
	 * line processor.
	 */
	public void init(String screenName);
	
	/**
	 * Binds a context to the line processor.
	 * @param context The context to bind.
	 */
	public void bindContext(LineProcContext context);
	
	/**
	 * Handles an incomming IM. This method should generate a response IM,
	 * but that is not required.
	 * @param message The incomming message.
	 */
	public void handleIM(IM message);
	
	/**
	 * Handles an incomming warning.
	 * @param screenName The screen name doing the warning.
	 */
	public void handleWarn(String screenName);
	
	/**
	 * Shuts down the line processor. Used to kill any started threads.
	 */
	public void shutdown();
}
