package commLibs;
import org.nukesoft.bot2.commlib.CommLib;
import org.nukesoft.bot2.commlib.CommLibCallback;
import org.nukesoft.bot2.core.IM;
import org.nukesoft.nukeutils.VerboseConsole;

import JavaTOC.Chatable;
import JavaTOC.JavaTOC;
import java.io.IOException;
import java.util.StringTokenizer;

/*
 *  Created on Oct 14, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/**
 * AIMLib implements the AOL Instant Messanger protocal within the specification
 * of the CommLib interface. This class relies on a repackaged version of the JavaTOC
 * library written by Jeff Heaton (<a href="http://www.jeffheaton.com">
 * http://www.jeffheaton.com</a>).
 * 
 * @author Ryan Michela
 */
public class AIMLib extends Thread implements CommLib, Chatable
{
	private CommLibCallback callback;
	private JavaTOC TOC = new JavaTOC(this);
	private String warningLevel;
	private String myScreenName;
	private String password;
	private boolean gotPing;
	
	public AIMLib()
	{
		super("AIMLib");
	}
	
	/**
	 * Provodes callback binding.
	 * @param callback The callback object to bind to.
	 * @see org.nukesoft.bot2.commlib.CommLib#bindCallback(org.nukesoft.bot2.commlib.CommLibCallback)
	 */
	public void bindCallback(CommLibCallback callback)
	{
		this.callback = callback;
	}

	/**
	 * Establishes a connection with the TOC server and attempts to start the dispatch
	 * thread.
	 * @param loggin The screen name to connect to.
	 * @param password The password to use.
	 * @see org.nukesoft.bot2.commlib.CommLib#connect(java.lang.String, java.lang.String)
	 */
	public boolean connect(String loggin, String password)
	{
		try
		{
			if(TOC.login(loggin, password))
			{
				//Connected! Start the dispatch thread.
				VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									   "AIMLib" +
									   VerboseConsole.GRAY_TEXT() +
									   ": Successfull connection to TOC server.", 
									   VerboseConsole.LOW);
				
				this.start();
				this.myScreenName = loggin;
				this.password = password;
				return true;
			}
			else
			{
				VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									   "AIMLib" +
									   VerboseConsole.GRAY_TEXT() +
								       ": Connection to TOC server FAILED!", 
									   VerboseConsole.LOW);
				return false;
			}
		}
		catch(IOException e)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								   "AIMLib" +
								   VerboseConsole.GRAY_TEXT() +
							  	   ": Connection to TOC server FAILED!", 
								   VerboseConsole.LOW);
			return false;
		}
	}

	/**
	 * Disconnects from the TOC network.
	 * @see org.nukesoft.bot2.commlib.CommLib#disconnect()
	 */
	public void disconnect()
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Disconnecting from TOC server.", 
								VerboseConsole.LOW);
		TOC.logout();
	}

	/**
	 * Sends a message.
	 * @param message The message to send.
	 * @see org.nukesoft.bot2.commlib.CommLib#sendMessage(org.nukesoft.bot2.core.IM)
	 */
	public void sendMessage(IM message)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Sending IM.", 
								VerboseConsole.LOW);
		String sourceString = message.message.replace('[', '{').replace(']', '}');
		
		TOC.send(message.screenName, sourceString);
	}

	/** Sends a warning.
	 * @param screenName The screen name to warn.
	 * @param anonymous Warn anonymously if true.
	 * @see org.nukesoft.bot2.commlib.CommLib#sendWarning(java.lang.String, boolean)
	 */
	public void sendWarning(String screenName, boolean anonymous)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() + 
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Sending warning.", 
								VerboseConsole.LOW);
		TOC.warn(screenName, anonymous);
	}

	/**
	 * Identifies the protocal this library implements.
	 * @see org.nukesoft.bot2.commlib.CommLib#identifyProtocal()
	 */
	public String identifyProtocal()
	{
		return "AIM";
	}

	/**
	 * Called by the connection port maintnance thread. Attempts to send a self
	 * addressed message. If the message does not return in three seconds, the
	 * connection is closed and reopened.
	 * @see org.nukesoft.bot2.commlib.CommLib#doPing()
	 */
	public synchronized void doPing()
	{
		gotPing = false;
		
		//send a self addressed IM
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() + 
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Sending self addressed ping.", 
								VerboseConsole.LOW);
		TOC.send(myScreenName, "PING!");
		
		//wait for three seconds, then check if the ping arrived.
		try
		{
			wait(3000);
		}
		catch(InterruptedException e)
		{
			//do nothing
		}
		
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Ping wait over.", 
								VerboseConsole.LOW);
		
		if(gotPing == true)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"AIMLib" +
									VerboseConsole.GRAY_TEXT() +
									": Ping success.", 
									VerboseConsole.LOW);
			gotPing = false;
		}
		else
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"AIMLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failed ping. Attempting to reconnect.", 
									VerboseConsole.HIGH);
			disconnect();
			TOC = new JavaTOC(this);
			connect(myScreenName, password);
		}
	}

	/**
	 * Handles incomming errors from the TOC library.
	 * @param str The ID of the error.
	 * @param var Not sure.
	 * @see JavaTOC.Chatable#error(java.lang.String, java.lang.String)
	 */
	public void error(String str, String var)
	{
		int errNum = Integer.parseInt(str);
		String errDesc = err(errNum);
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Error " + errNum + " / " + errDesc, 
								VerboseConsole.LOW);
		callback.handleError(errNum + " / " + errDesc);
		// TODO Improve error handleing?
	}

	/**
	 * Forewards an incomming IM throught the callback chain.
	 * @see JavaTOC.Chatable#im(java.lang.String, java.lang.String)
	 */
	public synchronized void im(String from, String message)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Incomming IM.", 
								VerboseConsole.LOW);
		if(from.toUpperCase().equals(myScreenName.toUpperCase()))
		{
			gotPing = true;
			notifyAll();
		}
		else
		{	
			callback.handleIM(new IM(from, dehtmlify(message)));
	
		}
	}

	/**
	 * Handles incomming unknown TOC events.
	 * @see JavaTOC.Chatable#unknown(java.lang.String)
	 */
	public void unknown(String str)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Incomming unknown event: " + str, 
								VerboseConsole.LOW);
		if(str.charAt(3) != '2')
		{
			StringTokenizer st = new StringTokenizer(str, ":");
			String unknown = st.nextToken();
			if(unknown.equals("UPDATE_BUDDY"))
			{
				st.nextToken();
				st.nextToken();
				warningLevel = st.nextToken();
				VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
										"AIMLib" +
										VerboseConsole.GRAY_TEXT() +
										": Warning level update: " + warningLevel + "%", 
										VerboseConsole.LOW);
			}
			else if(unknown.equals("EVILED"))
			{
				st.nextToken();
				if(st.hasMoreTokens())
				{
					String name = st.nextToken();
					VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
											"AIMLib" +
											VerboseConsole.GRAY_TEXT() +
											": Warn detected.", 
											VerboseConsole.LOW);
					callback.handleWarn(name);
				}
			}
		}
	}
	
	/**
	 * Gets the warning level.
	 * @see org.nukesoft.bot2.commlib.CommLib#getWarningLevel()
	 */
	public String getWarningLevel()
	{
		return warningLevel;
	}

	/**
	 * TOC Dispatch thread.
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Starting TOC dispatch thread.", 
								VerboseConsole.LOW);
		try
		{
			TOC.processTOCEvents();
		}
		catch(IOException e)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"AIMLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failure in TOC dispatch thread!", 
									VerboseConsole.LOW);
		}
	}

	
	/**
	 * Strips input of all HTML tags.
	 * @param input Text with embeded HTML.
	 * @return Text without any HTML.
	 */
	protected String dehtmlify(String input)
	{
	  String ret = new String();
	  boolean c = true;
	  for(int i = 0; i < input.length(); i++)
	  {
		if(c)
		{
		 if (input.charAt(i) == '<')
		 {
		   c = false;
		 }
		 else
		 {
		   ret += input.charAt(i);
		 }
		}
		else
		{
		  if(input.charAt(i) == '>')
		  {
			c = true;
		  }
		}
	  }
	  return ret.replaceAll("&quot;", "\"").replaceAll("&amp;", "&");
	}
	
	/**
	 * Converts a TOC error code into a TOC error string.
	 * @param num Error code.
	 * @return Error description.
	 */
	protected String err(int num)
	{
		{
		   String error;

		   switch ( num ) {
		   case 901:error="User not currently available";break;
		   case 902:error="Warning of user not currently available";break;
		   case 903:error="A message has been dropped, you are exceeding the server speed limit";break;
		   case 911:error="Error validating input";break;
		   case 912:error="Invalid account";break;
		   case 913:error="Error encountered while processing request";break;
		   case 914:error="Service unavailable";break;
		   case 950:error="Chat is unavailable.";break;
		   case 960:error="You are sending message too fast.";break;
		   case 961:error="You missed an im because it was too big.";break;
		   case 962:error="You missed an im because it was sent too fast.";break;
		   case 970:error="Failure";break;
		   case 971:error="Too many matches";break;
		   case 972:error="Need more qualifiers";break;
		   case 973:error="Dir service temporarily unavailable";break;
		   case 974:error="Email lookup restricted";break;
		   case 975:error="Keyword Ignored";break;
		   case 976:error="No Keywords";break;
		   case 977:error="Language not supported";break;
		   case 978:error="Country not supported";break;
		   case 979:error="Failure unknown";break;
		   case 980:error="Incorrect nickname or password.";break;
		   case 981:error="The service is temporarily unavailable.";break;
		   case 982:error="Your warning level is currently too high to sign on.";break;
		   case 983:error="You have been connecting and disconnecting too frequently.\nWait 10 minutes and try again.\n If you continue to try, you will need to wait even longer.";break;
		   case 989:error="An unknown signon error has occurred";break;
		   default:error="Unknown";break;

		   }
		   return error;
		}
	}
}
