import org.nukesoft.bot2.core.IM;
import org.nukesoft.bot2.lineproc.GenericLineProc;
import org.nukesoft.nukeutils.*;
import java.io.*;
import java.util.*;

/*
 *  Created on Nov 20, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/**
 * The AdminLineProc is a generic administration line processor for the bot
 * core. It provides remote access to most functions of the line proc context.
 * 
 * @author Ryan Michela
 */
public class AdminLineProc extends GenericLineProc
{
	String password;
	boolean locked = false;

	/**
	 * @see org.nukesoft.bot2.lineproc.LineProc#init(java.lang.String)
	 */
	public void init(String screenName)
	{
		super.init(screenName);
		context.sendIM(new IM(screenName, "=== Admin Menu === <br> Type 'help' for more info."));
	
		//read configuration
		try
		{
			XmlConfManager conf = new XmlConfManager("conf/AdminConf.xml");
			password = conf.getOneConfString("/Admin/Password");
		}
		catch(IOException e)
		{
			VerboseConsole.println(VerboseConsole.RED_TEXT() +
									"AdminLineProc" +
									VerboseConsole.GRAY_TEXT() +
									": Configuration error: " + e.getMessage(),
									VerboseConsole.CRITICAL);
			password = "";
		}
		
		//display password prompt if necesary
		if(!password.equals(""))
		{
			locked = true;
			context.sendIM(new IM(screenName, "Please enter administrative password:"));
		}
	}

	/**
	 * @see org.nukesoft.bot2.lineproc.LineProc#handleIM(org.nukesoft.bot2.core.IM)
	 */
	public void handleIM(IM message)
	{
		ExtendedStringTokenizer st = new ExtendedStringTokenizer(message.message);
		
		//check for password lock.
		if(locked)
		{
			if(st.restTokens().equals(password))
			{
				locked = false;
				context.sendIM(new IM(message.screenName, "Admin menu unlocked."));
			}
			else
			{
				context.sendIM(new IM(message.screenName, "=== Invalid password. ===<br>Leaving admin menu."));
				context.loadNewLineProc(context.lastLoadedLineProc());
			}
		}
		//handle commands
		else
		{
			String command = st.nextToken();
			if(command.equalsIgnoreCase("HELP"))
			{
				String helpString = "=== Admin Help ===<br>" +
									"Place quotes (\") around screen names and bind IDs if they " +
									"contain spaces.<br><br>" +
									"ATTACK screenName [salutation [commPortBindID [lineProcBindID]]]<br>" +
									"Initiates a conversation with another screen" +
									"name with an optional salutation over an optional " +
									"specified comm port. If no comm port is used, the " +
									"current one is used. Salutations must be quoted " +
									"if they contain spaces.<br><br>" +
									"INTERJECT dialogID interjection<br>" +
									"Interjects text into a specified dialog.<br><br>" +
									"BIND dialogID<br>"+
									"Binds your dialog to another, allowing you to seec" +
									"whatever is sent to and from the other dialog.<br><br>" +
									"UNBIND dialogID<br>" +
									"Breaks a previously set up binding.<br><br>" +
									"LIST (dialogIDs | commPortBindIDs | lineProcBindIDs)<br>" +
									"Returns a listing of internal environment registries. " +
									"This information is used when executing other commands.<br><br>" +
									"LOAD lineProcBindID [dialogID]<br>" +
									"Loads a new line processor. If a dialogID is specified, " +
									"the line processor is loaded on that dialog instead of " +
									"the current one.<br><br>" +
									"EXIT<br>" + 
									"Exits the administrative menu.";
				context.sendIM(new IM(message.screenName, helpString));
			}
			else if(command.equalsIgnoreCase("ATTACK"))
			{
				String screenName;
				String salutation = "Hello.";
				String commPortBindID = super.myCommLibBindID;
				String lineProcBindID = "";
				
				//gather parameters
				if(st.hasMoreTokens())
				{
					screenName = st.nextQuotedToken();
					
					if(st.hasMoreTokens())
					{
						salutation = st.nextQuotedToken();
						
						if(st.hasMoreTokens())
						{
							commPortBindID = st.nextQuotedToken();
							if(st.hasMoreTokens())
							{
								lineProcBindID = st.nextQuotedToken();
							}
						}
					}
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Malformed expression!"));
					return;
				}
				
				//send the attack
				if(context.getComPortBindIDs().contains(commPortBindID))
				{	
					if(lineProcBindID.equals(""))
					{	
						context.initiateConversation(commPortBindID, screenName, salutation);
						context.sendIM(new IM(message.screenName, "Attack successfull."));
					}
					else
					{
						if(context.getLineProcBindIDs().contains(lineProcBindID))
						{
							context.initiateConversation(commPortBindID, screenName, salutation);
							context.loadNewLineProcOnPeer(commPortBindID + "~" + screenName, lineProcBindID);
							context.sendIM(new IM(message.screenName, "Attack successfull."));
						}
						else
						{
							context.sendIM(new IM(message.screenName, "Attack failed. Invalid lineProcBindID."));
						}
					}
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Attack failed. Invalid commPortBindID."));
				}
			}
			else if(command.equalsIgnoreCase("INTERJECT"))
			{
				String dialogID;
				String interjection;
				
				//get parameters
				if(st.hasMoreTokens())
				{
					dialogID = st.nextQuotedToken();
					if(st.hasMoreTokens())
					{
						interjection = st.restTokens();
					}
					else
					{
						context.sendIM(new IM(message.screenName, "Malformed expression!"));
						return;
					}
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Malformed expression!"));
					return;
				}
				
				//send the interjection
				if(context.getConversations().contains(dialogID))
				{	
					context.interject(dialogID, interjection);
					context.sendIM(new IM(message.screenName, "Interjection successfull."));
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Interjection failed. Invalid dialogID."));
				}
			}
			else if(command.equalsIgnoreCase("BIND"))
			{
				String dialogBindID;
				
				//get parameters
				if(st.hasMoreTokens())
				{
					dialogBindID = st.nextQuotedToken();
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Malformed expression!"));
					return;
				}
				
				//execute the bind.
				if(context.bindConversation(dialogBindID))
				{
					context.sendIM(new IM(message.screenName, context.getIdentity() + " ===> " + dialogBindID));
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Bind failed."));
				}
			}
			else if(command.equalsIgnoreCase("UNBIND"))
			{
				String dialogBindID;
				
				//get parameters
				if(st.hasMoreTokens())
				{
					dialogBindID = st.nextQuotedToken();
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Malformed expression!"));
					return;
				}
				
				//execute the bind.
				if(context.unbindConversation(dialogBindID))
				{
					context.sendIM(new IM(message.screenName, context.getIdentity() + " =x=> " + dialogBindID));
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Unbind failed."));
				}
			}
			else if(command.equalsIgnoreCase("LIST"))
			{
				String mode;
				Set data;
				
				//determine the mode
				if(st.hasMoreTokens())
				{
					mode = st.nextToken();
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Malformed expression!"));
					return;
				}
				
				//execute the command
				if(mode.equalsIgnoreCase("dialogIDs"))
				{
					data = context.getConversations();
				}
				else if(mode.equalsIgnoreCase("commPortBindIDs"))
				{
					data = context.getComPortBindIDs();
				}
				else if(mode.equalsIgnoreCase("lineProcBindIDs"))
				{
					data = context.getLineProcBindIDs();
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Malformed expression!"));
					return;	
				}
				
				//return the set
				Iterator it = data.iterator();
				String conv = "";
				while(it.hasNext())
				{
					conv += (String)it.next();
					conv += "<br>";
				}
				context.sendIM(new IM(message.screenName, conv));
			}
			else if(command.equalsIgnoreCase("LOAD"))
			{
				String lineProcBindID;
				String dialogID = "";
				
				//load the parameters
				if(st.hasMoreTokens())
				{
					lineProcBindID = st.nextQuotedToken();
					if(st.hasMoreTokens())
					{
						dialogID = st.nextQuotedToken();
					}
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Malformed expression!"));
					return;
				}
				
				//execute the command
				if(context.getLineProcBindIDs().contains(lineProcBindID))
				{	
					if(dialogID.equals(""))
					{
						context.loadNewLineProc(lineProcBindID);
					}
					else
					{
						context.loadNewLineProcOnPeer(dialogID, lineProcBindID);
					}
					context.sendIM(new IM(message.screenName, "Load successfull."));
				}
				else
				{
					context.sendIM(new IM(message.screenName, "Load failed. Invalid lineProcBindID."));
				}
			}
			else if(command.equalsIgnoreCase("EXIT"))
			{
				context.sendIM(new IM(message.screenName, "=== Leaving Admin Menu ==="));
				context.loadNewLineProc(context.lastLoadedLineProc());
			}
			else
			{
				context.sendIM(new IM(message.screenName, "Unknown command. Type HELP for a list of commands."));
				return;
			}
		}
	}

}
