package org.nukesoft.bot2.commlib.eliza;
import org.nukesoft.bot2.core.IM;
import org.nukesoft.bot2.lineproc.GenericLineProc;

/*
 *  Created on Nov 19, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

import java.io.*;
import net.chayden.eliza.*;
import org.txml.*;
import org.nukesoft.nukeutils.*;

/**
 * Description goes here.
 * 
 * @author Ryan Michela
 */
public class ElizaLineProc extends GenericLineProc
{
	/**
	 * The Eliza processor for this session
	 */
	private ElizaProc eliza;
	private String scriptPath = "conf/ElizaScript";
	private String admin = "";
	private String escape = "#!";
	private int minDelay = 5;
	private int maxDelay = 10;
	private int warnCount = 0;
	
	/**
	 * Builds an ElizaLineProc. Loads settings from xml config file.
	 * Uses defaults if config file cannot be found.
	 */
	public ElizaLineProc()
	{
		//load the xml conf file
		File conf = new File("conf/ElizaConf.xml");
		if(conf.exists())
		{
			try
			{
				XmlConfManager root = new XmlConfManager(conf);
				scriptPath = root.getOneConfString("/Eliza/ScriptPath");
				admin = root.getOneConfString("/Eliza/AdminLineProcBindName");
				escape = root.getOneConfString("/Eliza/AdminEscapeSequence");
				minDelay = Integer.parseInt(root.getOneConfString("/Eliza/MinResponseDelaySecs"));
				maxDelay = Integer.parseInt(root.getOneConfString("/Eliza/MaxResponseDelaySecs"));
			}
			catch(IOException e)
			{
				VerboseConsole.println(VerboseConsole.RED_TEXT() +
										"ElizaLineProc" +
										VerboseConsole.GRAY_TEXT() +
										": Error loading ElizaConf.xml! Using default configuration.", 
										VerboseConsole.CRITICAL);
			}
			catch(XMLSyntaxError e)
			{
				VerboseConsole.println(VerboseConsole.RED_TEXT() +
										"ElizaLineProc" +
										VerboseConsole.GRAY_TEXT() +
										": Error parsing ElizaConf.xml! Using default configuration.", 
										VerboseConsole.CRITICAL);
			}
			catch(NumberFormatException e)
			{
				VerboseConsole.println(VerboseConsole.RED_TEXT() +
										"ElizaLineProc" +
										VerboseConsole.GRAY_TEXT() +
										": Error parsing ElizaConf.xml! Using default configuration.", 
										VerboseConsole.CRITICAL);
			}	
		}
		else
		{
			VerboseConsole.println(VerboseConsole.RED_TEXT() +
									"ElizaLineProc" +
									VerboseConsole.GRAY_TEXT() +
									": ElizaConf.xml does not exist! Using default configuration.", 
									VerboseConsole.CRITICAL);
		}
		
		//create the line processor
		eliza = new ElizaProc(scriptPath);
	}
	
	/**
	 * Handles an incomming IM, Eliza style
	 * @see org.nukesoft.bot2.lineproc.LineProc#handleIM(org.nukesoft.bot2.core.IM)
	 */
	public void handleIM(IM message)
	{		
		String m = "";
		try
		{	
			//delay for a sec
			delay(minDelay, maxDelay);
			//send an eliza message
			m = eliza.mutate(message.message);
		}
		catch(Exception e)
		{	
			eliza = new ElizaProc(scriptPath);
			m = eliza.mutate(message.message);
		}
		finally
		{
			context.sendIM(new IM(message.screenName,m));
		}
	}

	/**
	 * Handles an incomming warning, Eliza style
	 * @see org.nukesoft.bot2.lineproc.LineProc#handleWarn(java.lang.String)
	 */
	public void handleWarn(String screenName)
	{
		warnCount++;
		
		//delay for a sec
		delay(minDelay, maxDelay);
		
		//send response
		if(warnCount == 1)
		{
			context.sendIM(new IM(screenName, "Now that wasn't very nice. :("));
		}
		else if(warnCount == 2)
		{
			context.sendIM(new IM(screenName, "Cut that out!"));
			context.sendWarning(screenName, true);
		}
		else
		{
			context.sendIM(new IM(screenName, "HEY!"));
			context.sendWarning(screenName, false);
		}
	}

}
