/*
 *  Created on Dec 2, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package commLibs;
import org.nukesoft.bot2.commlib.*;
import org.nukesoft.bot2.core.IM;
import org.nukesoft.nukeutils.*;
import org.hn.sleek.jmml.*;

/**
 * The MSNLib class implements the MSN communication protocal within the
 * CommLib specification. It uses the jmml MSN library by Tony Tang 
 * (<a href="http://www.sfu.ca/~tonyt/">http://www.sfu.ca/~tonyt/</a>.
 * 
 * @author Ryan Michela
 */
public class MSNLib extends MessengerClientAdapter implements CommLib 
{
	private CommLibCallback callback;
	private MessengerServerManager MSN;
	private String warningLevel;
	private String myScreenName;
	private String password;
	private boolean gotPing;

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#bindCallback(org.nukesoft.bot2.commlib.Cfck)
	 */
	public void bindCallback(CommLibCallback callback)
	{
		this.callback = callback;
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#connect(java.lang.String, java.lang.String)
	 */
	public boolean connect(String loggin, String password)
	{
		MSN = MessengerServerManager.getInstance();

		MSN.signIn(loggin, password, ContactStatus.ONLINE, true);

		if(!MSN.isConnected())
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"MSNLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failed to connect to MSN. ");
			return false;
		}
		
		MSN.addMessengerClientListener(this);
		this.myScreenName = loggin;
		this.password = password;
		return true;
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#disconnect()
	 */
	public void disconnect()
	{
		MSN.signOut();
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#doPing()
	 */
	public void doPing()
	{
		if(!MSN.isConnected())
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"MSNLib" +
									VerboseConsole.GRAY_TEXT() +
									": Ping failed, reconnecting. ");
			MSN.signOut();
			connect(myScreenName, password);
		}
		else
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"MSNLib" +
									VerboseConsole.GRAY_TEXT() +
									": Ping success.", 
									VerboseConsole.LOW);
		}
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#getWarningLevel()
	 */
	public String getWarningLevel()
	{
		return "N/A";
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#identifyProtocal()
	 */
	public String identifyProtocal()
	{
		return "MSN";
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#sendMessage(org.nukesoft.bot2.core.IM)
	 */
	public void sendMessage(IM message)
	{
		MSN.sendMessage(message.screenName, message.message);
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#sendWarning(java.lang.String, boolean)
	 */
	public void sendWarning(String screenName, boolean anonymous)
	{
		//Not supported by MSN protocal.
	}

	/* (non-Javadoc)
	 * @see org.hn.sleek.jmml.MessengerClientListener#incomingMessage(org.hn.sleek.jmml.IncomingMessageEvent)
	 */
	public void incomingMessage(IncomingMessageEvent inMsg)
	{
		VerboseConsole.println(VerboseConsole.BLUE_BACK() +
								"MSNLib" +
								VerboseConsole.GRAY_TEXT() +
								": Incomming IM.", 
								VerboseConsole.LOW);
		IM outMsg = new IM(inMsg.getUserName(), stripFormat(inMsg.getMessage()));
		callback.handleIM(outMsg);
	}

	private String stripFormat(String in)
	{
		return in;
	}
}
