package commLibs;
import org.nukesoft.bot2.commlib.CommLib;
import org.nukesoft.bot2.commlib.CommLibCallback;
import org.nukesoft.bot2.core.IM;
import org.nukesoft.nukeutils.*;

import ymsg.network.*;
import ymsg.network.event.*;

import java.io.*;

/*
 *  Created on Nov 12, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/**
 * yahooLib implements the Yahoo Messanger protocal within the specification
 * of the CommLib interface. This class relies on the jYMSG
 * library found on Sourceforge (<a href="http://sourceforge.net/projects/jymsg9/">
 * http://sourceforge.net/projects/jymsg9/</a>).
 * 
 * @author Ryan Michela
 */
public class YahooLib extends SessionAdapter implements CommLib
{
	private CommLibCallback callback;
	private Session session = new Session();
	private String loggin;
	private String password;
	private boolean gotPing;
	
	public YahooLib()
	{
		session.addSessionListener(this);
	}
	
	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#bindCallback(org.nukesoft.bot2.commlib.CommLibCallback)
	 */
	public void bindCallback(CommLibCallback callback)
	{
		this.callback = callback;
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#connect(java.lang.String, java.lang.String)
	 */
	public boolean connect(String loggin, String password)
	{
		this.loggin = loggin;
		this.password = password;
		try
		{
			session.login(loggin, password);
			if(session.getSessionStatus() == Session.MESSAGING)
			{
				VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
										"YahooLib" +
										VerboseConsole.GRAY_TEXT() +
										": Successfull connection to Yahoo server.", 
										VerboseConsole.LOW);
				return true;
			}
			else
			{
				VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
										"YahooLib" +
										VerboseConsole.GRAY_TEXT() +
										": Failed to connect to Yahoo server!", 
										VerboseConsole.LOW);
				session.reset();
				return false;
			}
		}
		catch(IllegalStateException e)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failed to connect to Yahoo server. Allready connected!", 
									VerboseConsole.LOW);
			return false;
		}
		catch(IOException e)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failed to connect to Yahoo server!", 
									VerboseConsole.LOW);
			return false;
		}
		catch(AccountLockedException e)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failed to connect to Yahoo server. Account locked!", 
									VerboseConsole.LOW);
			return false;
		}
		catch(LoginRefusedException e)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failed to connect to Yahoo server. Loggin refused!", 
									VerboseConsole.LOW);
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#disconnect()
	 */
	public void disconnect()
	{
		try
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": Disconnecting from Yahoo server.", 
									VerboseConsole.LOW);
			session.logout();
		}
		catch(Exception e)
		{
			
		}
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#sendMessage(org.nukesoft.bot2.core.IM)
	 */
	public void sendMessage(IM message)
	{
		try
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": Sending IM.", 
									VerboseConsole.LOW);
			session.sendMessage(message.screenName, message.message);
		}
		catch(Exception e)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": IM send failed for some reason.", 
									VerboseConsole.LOW);
		}
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#sendWarning(java.lang.String, boolean)
	 */
	public void sendWarning(String screenName, boolean anonymous)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"YahooLib" +
								VerboseConsole.GRAY_TEXT() +
								": Yahoo protocal does not support warnings.", 
								VerboseConsole.LOW);
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#getWarningLevel()
	 */
	public String getWarningLevel()
	{
		return "N/A";
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#identifyProtocal()
	 */
	public String identifyProtocal()
	{
		return "Yahoo";
	}

	/* (non-Javadoc)
	 * @see org.nukesoft.bot2.commlib.CommLib#doPing()
	 */
	public void doPing()
	{
		gotPing = false;
		
		//send a self addressed IM
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() + 
								"AIMLib" +
								VerboseConsole.GRAY_TEXT() +
								": Sending self addressed ping.", 
								VerboseConsole.LOW);
		sendMessage(new IM(loggin, "PING!"));
		
		//wait for three seconds, then check if the ping arrived.
		try
		{
			synchronized(this)
			{
				wait(3000);
			}
		}
		catch(InterruptedException e)
		{
			//do nothing
		}
		
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"YahooLib" +
								VerboseConsole.GRAY_TEXT() +
								": Ping wait over.", 
								VerboseConsole.LOW);
		
		if(gotPing == true)
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"YahooLib" +
									VerboseConsole.GRAY_TEXT() +
									": Ping success.", 
									VerboseConsole.LOW);
			gotPing = false;
		}
		else
		{
			VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
									"AIMLib" +
									VerboseConsole.GRAY_TEXT() +
									": Failed ping. Attempting to reconnect.", 
									VerboseConsole.HIGH);
			disconnect();
			session.reset();
			connect(loggin, password);
		}
	}

	/* (non-Javadoc)
	 * @see ymsg.network.event.SessionListener#buzzReceived(ymsg.network.event.SessionEvent)
	 */
	public void buzzReceived(SessionEvent msg)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"YahooLib" +
								VerboseConsole.GRAY_TEXT() +
								": Incomming IM.", 
								VerboseConsole.LOW);
		callback.handleIM(new IM(msg.getFrom(), msg.getMessage()));
	}

	/* (non-Javadoc)
	 * @see ymsg.network.event.SessionListener#messageReceived(ymsg.network.event.SessionEvent)
	 */
	public void messageReceived(SessionEvent msg)
	{
		VerboseConsole.println(VerboseConsole.BLUE_BACK() +
								"YahooLib" +
								VerboseConsole.GRAY_TEXT() +
								": Incomming IM.", 
								VerboseConsole.LOW);
		if(msg.getFrom().toUpperCase().equals(loggin.toUpperCase()))
		{
			gotPing = true;
			synchronized(this)
			{
				notifyAll();
			}
		}
		else
		{	
			callback.handleIM(new IM(msg.getFrom(), deFormat(msg.getMessage())));
		}
	}

	/* (non-Javadoc)
	 * @see ymsg.network.event.SessionListener#errorPacketReceived(ymsg.network.event.SessionErrorEvent)
	 */
	public void errorPacketReceived(SessionErrorEvent e)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"YahooLib" +
								VerboseConsole.GRAY_TEXT() +
								": Error! " + e.getCode(), 
								VerboseConsole.LOW);
	}
	
	/* (non-Javadoc)
	 * @see ymsg.network.event.SessionListener#inputExceptionThrown(ymsg.network.event.SessionExceptionEvent)
	 */
	public void inputExceptionThrown(SessionExceptionEvent e)
	{
		VerboseConsole.println(VerboseConsole.BLUE_TEXT() +
								"YahooLib" +
								VerboseConsole.GRAY_TEXT() +
								": Error in the input thread! " + e.getException(), 
								VerboseConsole.LOW);
	}
	
	private String deFormat(String in)
	{
		StringBuffer out = new StringBuffer();
		boolean inEscape = false;
		
		for(int i = 0; i < in.length(); i++)
		{
			char c = in.charAt(i);
			
			if(inEscape)
			{
				if(c == 'm')
				{
					inEscape = false;
				}
			}
			else
			{	
				if (c == '[')
				{
					inEscape = true;
				}
				else
				{
					out.append(c);
				}
			}
		}
		return dehtmlify(out.toString());
	}
	
	private String dehtmlify(String input)
	{
	  String ret = new String();
	  boolean c = true;
	  for(int i = 0; i < input.length(); i++)
	  {
		if(c)
		{
		 if (input.charAt(i) == '<')
		 {
		   c = false;
		 }
		 else
		 {
		   ret += input.charAt(i);
		 }
		}
		else
		{
		  if(input.charAt(i) == '>')
		  {
			c = true;
		  }
		}
	  }
	  return ret.replaceAll("&quot;", "\"").replaceAll("&amp;", "&");
	}
}
