/*
 *  Created on Oct 31, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.nukesoft.bot2;
import java.io.*;
import org.nukesoft.bot2.core.*;
import org.nukesoft.nukeutils.*;

/**
 * Description goes here.
 * 
 * @author Ryan Michela
 */
public class Bot2 extends Thread
{
	public static void main(String[] args)
	{
		new Bot2(args);
	}
	
	public Bot2(String[] args)
	{
		this.setDaemon(true);
		
		//Start the bot
		DialogManager dm = new DialogManager(buildDMConf(args));
		VerboseConsole.println(VerboseConsole.GREEN_TEXT() +
							   "Driver" +
							   VerboseConsole.GRAY_TEXT() +
								     ": Bot successfully loaded.", VerboseConsole.CRITICAL);
		VerboseConsole.println("        Type '" + VerboseConsole.MAGENTA_TEXT() + "Q" + VerboseConsole.GRAY_TEXT() +"' to quit.", VerboseConsole.CRITICAL);
		VerboseConsole.println("        Type '" + VerboseConsole.MAGENTA_TEXT() + "V #" + VerboseConsole.GRAY_TEXT() +"' to reset verbosity.", VerboseConsole.CRITICAL);
		VerboseConsole.println("        Type '" + VerboseConsole.MAGENTA_TEXT() + "R" + VerboseConsole.GRAY_TEXT() +"' to reboot the bot core.", VerboseConsole.CRITICAL);
		
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		try
		{
			String line;
			while((line = br.readLine()) != null)
			{	
				ExtendedStringTokenizer st = new ExtendedStringTokenizer(line);
				if(st.hasMoreTokens())
				{
					String cmd = st.nextToken();
					if(cmd.equalsIgnoreCase("Q"))
					{
						break;
					}
					else if(cmd.equalsIgnoreCase("V"))
					{
						if(st.hasMoreTokens())
						{
							String v = st.nextToken();
							setVerbosity(v);
						}
					}
					else if(cmd.equalsIgnoreCase("R"))
					{
						dm.shutdown();
						dm = new DialogManager(buildDMConf(args));
					}
				}
			}
		}
		catch(Exception e)
		{
			
		}
		
		VerboseConsole.println(VerboseConsole.GREEN_TEXT() +
								"Driver" +
								VerboseConsole.GRAY_TEXT() +
								": Stopping bot...", 
								VerboseConsole.CRITICAL);
		this.start();
		dm.shutdown();
	}
	
	private void usageAndQuit()
	{
		VerboseConsole.println("usage:  " + VerboseConsole.BOLD_ON() +"Bot2" + VerboseConsole.BOLD_OFF() + " [-c] [-C path] [-V {0..4}]", VerboseConsole.CRITICAL);
		VerboseConsole.println("", VerboseConsole.CRITICAL);
		VerboseConsole.println("  " + VerboseConsole.BOLD_ON() +"c" + VerboseConsole.BOLD_OFF() + " : toggle ANSI color output", VerboseConsole.CRITICAL);
		VerboseConsole.println("", VerboseConsole.CRITICAL);
		VerboseConsole.println("  " + VerboseConsole.BOLD_ON() +"C" + VerboseConsole.BOLD_OFF() + " path  : use an alternate configuration file located at 'path'", VerboseConsole.CRITICAL);
		VerboseConsole.println("  " + VerboseConsole.BOLD_ON() +"V" + VerboseConsole.BOLD_OFF() + " level : sepcify a verbosity level. 0 - silent, 4 - extremely verbose", VerboseConsole.CRITICAL);
		System.exit(0);
	}
	
	private void setVerbosity(String v)
	{
		if(v.equals("0"))
		{
			VerboseConsole.setLevel(VerboseConsole.SILENT);
		}
		else if(v.equals("1"))
		{
			VerboseConsole.setLevel(VerboseConsole.CRITICAL);
		}
		else if(v.equals("2"))
		{
			VerboseConsole.setLevel(VerboseConsole.HIGH);
		}
		else if(v.equals("3"))
		{
			VerboseConsole.setLevel(VerboseConsole.MEDIUM);
		}
		else if(v.equals("4"))
		{
			VerboseConsole.setLevel(VerboseConsole.LOW);
		}
	}
	
	private DMConfig buildDMConf(String[] args)
	{
		//establish command line defaults
		String confPath = "conf/BotConf.xml";
		String verbosity = "-1";
		boolean ansi = false;
		
		//process the command line
		try
		{
			for(int i = 0; i < args.length; i++)
			{
				if(args[i].equals("-c"))
				{
					ansi = true;
				}
				else if(args[i].equals("-C"))
				{
					confPath = args[++i];
				}
				else if(args[i].equals("-V"))
				{
					verbosity = new Integer(args[++i]).toString();
				}
				else
				{
					usageAndQuit();
				}
			}
		}
		catch(Exception e)
		{
			usageAndQuit();
		}
		
		//load the config file
		DMConfig dmConf = null;
		try
		{
			XmlConfManager confRoot = null;
			
			confRoot = new XmlConfManager(confPath);

		
			//read ansi and verbosity settings
			if(confRoot.getOneConfString("/Bot/AnsiColor").equalsIgnoreCase("TRUE"))
			{
				ansi = !ansi;
			}
			VerboseConsole.enableANSI(ansi);
			if(verbosity.equals("-1"))
			{	
				verbosity = confRoot.getOneConfString("/Bot/DefaultVerbosity");
			}
			if(verbosity.equals("0") || 
					verbosity.equals("1") || 
					verbosity.equals("2") || 
					verbosity.equals("3") || 
					verbosity.equals("4"))
			{
				setVerbosity(verbosity);
			}
			else
			{
				VerboseConsole.println(VerboseConsole.RED_TEXT() +
										"Invalid verbosity in config file!" +
										VerboseConsole.GRAY_TEXT(), 
										VerboseConsole.CRITICAL);
				System.exit(0);		
			}
			
			//build the DMConfig
			int dialogTimeout = Integer.parseInt(confRoot.getOneConfString("/Bot/DialogTimeout"));
			int logFlushDelay = Integer.parseInt(confRoot.getOneConfString("/Bot/LogFlushDelay"));
			String logWorkingDir = confRoot.getOneConfString("/Bot/LogWorkingDir");
			String adminLineProcBindID = confRoot.getOneConfString("/Bot/AdminLineProcBindID");
			String adminEscapeSequence = confRoot.getOneConfString("/Bot/AdminLineProcBindID/Escape");
			
			XmlConfManager[] cpNodes = confRoot.getConfSet("/Bot/CommPort");
			XmlConfManager[] lpNodes = confRoot.getConfSet("/Bot/LineProc");
			XmlConfManager[] dbNodes = confRoot.getConfSet("/Bot/DefaultBind");
			
			dmConf = new DMConfig(dialogTimeout, logFlushDelay, logWorkingDir, adminLineProcBindID, adminEscapeSequence);
			
			for(int i = 0; i < cpNodes.length; i++)
			{
				DMConfig.CPConfig c = dmConf.new CPConfig(cpNodes[i].getOneConfString("/CommPort/BindName"),
														  cpNodes[i].getOneConfString("/CommPort/LogIn"),
														  cpNodes[i].getOneConfString("/CommPort/Password"),
														  cpNodes[i].getOneConfString("/CommPort/LibPath"),
														  Integer.parseInt(cpNodes[i].getOneConfString("/CommPort/PingDelay")));
				dmConf.addCPConfig(c);
			}
			for(int i = 0; i < lpNodes.length; i++)
			{
				DMConfig.LPConfig c = dmConf.new LPConfig(lpNodes[i].getOneConfString("/LineProc/BindName"),
														  lpNodes[i].getOneConfString("/LineProc/LibPath"));
				dmConf.addLPConfig(c);
			}
			for(int i = 0; i < dbNodes.length; i++)
			{
				DMConfig.DBConfig c = dmConf.new DBConfig(dbNodes[i].getOneConfString("/DefaultBind/CommPortBindName"),
														  dbNodes[i].getOneConfString("/DefaultBind/LineProcBindName"));
				dmConf.addDBConfig(c);
			}
			
			return dmConf;
		}
		catch(IOException e)
		{
			VerboseConsole.println(VerboseConsole.RED_TEXT() +
									"Error in BotConf.xml " +
									VerboseConsole.GRAY_TEXT() +
									e.getMessage(), 
									VerboseConsole.CRITICAL);
			System.exit(0);
			return null;
		}		
	}
	
	public void run()
	{
		VerboseConsole.println(VerboseConsole.GREEN_TEXT() +
								"Shutting down in:" +
								VerboseConsole.GRAY_TEXT(), VerboseConsole.HIGH);
		for(int i = 10; i > 0; i--)
		{
			VerboseConsole.println(VerboseConsole.RED_TEXT() +
									i + "" +
									VerboseConsole.GRAY_TEXT(), 
									VerboseConsole.HIGH);
			try{sleep(1000);}catch(Exception e){}
		}
		System.exit(0);
	}
}
