/*
 *  Created on Oct 14, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.nukesoft.bot2.commlib;
import org.nukesoft.bot2.core.*;

/**
 * The CommLib interface describes the contract all communication libraries must
 * abide by. It provides a common interface for all IM services. 
 * 
 * @author Ryan Michela
 */
public interface CommLib 
{
	
	/**
	 * Binds the communication library to a callback object.
	 * @param callback
	 */
	public void bindCallback(CommLibCallback callback);
	
	/**
	 * Establishes a connection to the IM server.
	 * @param loggin
	 * @param password
	 * @return TRUE if successfull. FALSE if not.
	 */
	public boolean connect(String loggin, String password);
	/**
	 * Terminates connection to the IM server.
	 */
	public void disconnect();
	
	/**
	 * Sends an instant message over the network.
	 * @param message
	 */
	public void sendMessage(IM message);
	/**
	 * Attempts to warn an individual. May not have an effect on all networks.
	 * @param screenName The target.
	 * @param anonymous If true, attempt to warn anonymously.
	 */
	public void sendWarning(String screenName, boolean anonymous);
	
	/**
	 * Gets the current warning level of this account.
	 * @return "###" if warning level supported, "N/A" if not.
	 */
	public String getWarningLevel();
	
	/**
	 * Returns a string identifying the protocal this library implements.
	 * @return The protocal string.
	 */
	public String identifyProtocal();
	
	/**
	 * <p>Force the CommLib to ping its connection.</p>
	 * <p><b>This will always be called from the connection maintnance thread, thus,
	 * the doPing implementation must block until the action is complete.</b>
	 * The spirit of this method is to attempt to send a self addressed message,
	 * and reconnect to the network if the message fails to come through.</p>
	 */
	public void doPing();
}
