/*
 *  Created on Oct 14, 2003
 *  Copyright 2003 by Ryan Michela
 *
 *   This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.nukesoft.bot2.core;

import java.io.*;
import java.util.*;

/**
 * The AbsolutePathClassLoader is used to load classes based on their OS path, rather
 * than their package path. This allows the flexability to create a plug-in based
 * system where extension classes are dynamicaly loaded from a plug in dirrectory.
 * 
 * <p>Usage:<br>
 * AbsolutePathClassLoader apcl = new AbsolutePathClassLoader();<br>
 * Object o = (Object)apcl.loadClass("path").newInstance();
 *		  
 * @author Ryan Michela
 */

public class AbsolutePathClassLoader extends ClassLoader 
{
	/**
	 *This Map is used for cacheing. Classes are only loaded from the drive the first
	 *time they are retreived.
	 */
	private static HashMap classMap = new HashMap();
	
	/**
	 * @param The parent class loader. You should usualy use the default constructor.
	 */
	public AbsolutePathClassLoader(ClassLoader parent) 
	{
		super(parent);
	}
	
	public AbsolutePathClassLoader() 
	{
		super();
	}

	/**
	 * Finds the class in question.
	 * @see java.lang.ClassLoader#findClass(java.lang.String)
	 * @throws ClassNotFoundException
	 */
	public Class findClass(String name) throws ClassNotFoundException 
	{
		try
		{
			Class c = (Class)classMap.get(name);
			//check the cache
			if(c == null)
			{
				//locate the file
				File f = new File(name);
				FileInputStream fis = new FileInputStream(f);
				byte[] b = new byte[fis.available()];
				if(fis.read(b) == -1)
				{
					throw new Exception();
				}
				
				//extract the class name form the file path
				//yes, this is nasty character processing
				StringBuffer sb = new StringBuffer();
				char[] ch = name.toCharArray();
				boolean classPassed = false;
				for(int i = ch.length - 1; i >= 0; i--)
				{
					if(ch[i] == '/' || ch[i] == '\\' || ch[i] == ':') break;
					if(classPassed) sb.insert(0, ch[i]);
					if(ch[i] == '.') classPassed = true;
				}
				
				c = defineClass(sb.toString(), b, 0, b.length);
				resolveClass(c);
				
				//cache the class object
				classMap.put(name, c);
			}
			return c;
		}
		catch(Exception e)
		{
			throw new ClassNotFoundException();
		}
	}

}
